import os
import logging
from typing import Any, Dict, Optional

import yaml

_LOGGER = logging.getLogger(__name__)


class ConfigError(Exception):
    pass


class Config:
    def __init__(
        self,
        ggci_secret: str,
        user_mappings: Optional[Dict[int, int]] = None,
        **kwargs,
    ):
        """GGCI configuration.

        Can be loaded to the GGCI Flask app by using
        `app.config.from_object(dict(Config(...)))`.
        """
        if user_mappings is None:
            user_mappings = {}

        if not isinstance(ggci_secret, str):
            raise TypeError(
                f'ggci-secret must be of type str, got: {type(ggci_secret)}'
            )
        if not isinstance(user_mappings, dict):
            raise TypeError(
                f'user_mappings must be of type dict, got:'
                f' {type(user_mappings)}'
            )

        self._config_dict = {
            'GGCI_SECRET': ggci_secret,
            'GGCI_USER_MAPPINGS': user_mappings,
            **{key.upper(): val for key, val in kwargs.items()},
        }

    def __getitem__(self, key: str) -> Any:
        return self._config_dict[key.upper()]

    def __iter__(self):
        for key, val in self._config_dict.items():
            yield key, val

    def __len__(self):
        return len(self._config_dict)


def load_yaml_config(file_path: Optional[str] = None) -> Config:
    """Loads GGCI configuration from YAML file.

    :param file_path: Path to the YAML config file. If not specified,
     file path from environment variable GGCI_CONFIG is used.
    :type file_path: Optional[str], optional
    :return: GGCI configuration object
    :rtype: Config
    """

    if file_path is None:
        _LOGGER.info(
            'Config file path not specified, using env variable GGCI_CONFIG...'
        )
        try:
            file_path = os.environ['GGCI_CONFIG']
        except KeyError as exc:
            raise ConfigError(
                'Neither file_path argument was supplied nor GGCI_CONFIG'
                ' environment variable was found.'
                ' Cannot load configuration.'
            ) from exc

    _LOGGER.info('Loading GGCI config from file %s...', file_path)

    with open(file_path) as cfg_file:
        try:
            cfg_dict = yaml.safe_load(cfg_file)
        except yaml.YAMLError as exc:
            raise ConfigError('Error loading config file') from exc

    return Config(**cfg_dict)



import re
import secrets
from typing import Tuple

from flask import Blueprint, current_app, request

from ggci.google_chat import GoogleChatError, send_message
from ggci.gitlab import MergeRequestEvent, InvalidFormat, UnsupportedEvent

_GITLAB_TOKEN_REGEX_PATTERN = 'GGCI-SECRET=(.*);GOOGLE-CHAT-URL=(.*)'
_GOOGLE_CHAT_REGEX_PATTERN = 'https://chat.googleapis.com/v1/spaces/.*'
_UNAUTHORIZED_RESPONSE = ('Unauthorized', 401)

bp = Blueprint('forwarder', __name__)


class IncorrectTokenFormat(Exception):
    pass


class Unauthorized(Exception):
    pass


def _parse_gitlab_token(gitlab_token: str) -> Tuple[str, str]:
    if not isinstance(gitlab_token, str):
        raise IncorrectTokenFormat()
    match = re.match(_GITLAB_TOKEN_REGEX_PATTERN, gitlab_token)
    if match is None:
        raise IncorrectTokenFormat()
    ggci_secret, google_chat_url = match.groups()
    return ggci_secret, google_chat_url


def _authorize(ggci_secret: str) -> None:
    if not secrets.compare_digest(
        ggci_secret, current_app.config['GGCI_SECRET']
    ):
        raise Unauthorized()


@bp.route('/', methods=['POST'])
def forward():

    try:
        ggci_secret, google_chat_url = _parse_gitlab_token(
            gitlab_token=request.headers.get('X-Gitlab-Token'),
        )
    except IncorrectTokenFormat:
        return _UNAUTHORIZED_RESPONSE

    try:
        _authorize(ggci_secret=ggci_secret)
    except Unauthorized:
        return _UNAUTHORIZED_RESPONSE

    if re.fullmatch(_GOOGLE_CHAT_REGEX_PATTERN, google_chat_url) is None:
        return (
            f'Google Chat URL does not match the following regex pattern:'
            f' {_GOOGLE_CHAT_REGEX_PATTERN}'
        ), 400

    try:
        mr_event = MergeRequestEvent.from_dict(request.json)
    except InvalidFormat as exc:
        return str(exc), 400
    except UnsupportedEvent as exc:
        return str(exc), 501

    try:
        send_message(url=google_chat_url, message=mr_event.create_message())
    except GoogleChatError as exc:
        return str(exc), 500

    return 'Success', 200



import logging

import requests
import tenacity

from ggci.commons import Message

_LOGGER = logging.getLogger(__name__)


class GoogleChatError(Exception):
    def __init__(self, error):

        if isinstance(error, tenacity.Future):
            # Tenacity initiates `retry_error_cls` with tenacity.Future.
            # containing more information along with the exception itself
            error = error.exception()

        super().__init__(error)


@tenacity.retry(
    retry=tenacity.retry_if_exception_type(requests.exceptions.HTTPError),
    stop=tenacity.stop_after_attempt(5),
    wait=tenacity.wait.wait_random(min=0.05, max=0.2),
    retry_error_cls=GoogleChatError,
)
def send_message(url: str, message: Message) -> None:

    _LOGGER.info('Sending message...')
    _LOGGER.debug('Message: %s', message)

    if message.thread_key is not None:
        url += f'&threadKey=GGCI_{message.thread_key}'

    response = requests.post(url=url, json={'text': message.text})
    response.raise_for_status()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ggci']

package_data = \
{'': ['*']}

install_requires = \
['flask>=1.1.2,<3.0',
 'flexmock>=0.10.4,<0.11.0',
 'pyyaml>=5.4.1,<6.0.0',
 'requests>=2.25.1,<3.0.0',
 'tenacity>=7.0.0,<8.0.0']

setup_kwargs = {
    'name': 'ggci',
    'version': '1.2.17',
    'description': 'GitLab Google Chat Integration',
    'long_description': "# Gitlab Google Chat Integration\n\nA Python Flask web application that forwards webhook requests\nfrom GitLab to Google Chat.\n\n![GGCI showcase](https://raw.githubusercontent.com/lukany/ggci/cb0886eb6594e36c5e56e54f00dbfdb71d3d8629/showcase.png)\n\n## Installation\n\n`ggci` is available as a Python package\non [PyPI](https://pypi.org/project/ggci).\nIt can be installed via standard package managers in Python, e.g. pip:\n\n```sh\npip install ggci\n```\n\n## Usage\n\n`ggci` provides a standard Flask application factory `create_app()`:\n\n```python\nfrom ggci import create_app\n\napp = create_app()\n```\n\nFor how to use this application factory refer to the official [Flask\ndocumentation](https://flask.palletsprojects.com/en/1.1.x/).\n\n## Configuration\n\nThere are several ways how `ggci` can be configured.\n\n### YAML config (default)\n\nBy default `create_app()` looks for a YAML configuration file specified\nby `GGCI_CONFIG` environment variable.\nExample config:\n\n```YAML\nggci_secret: xxxxxxx\n\nuser_mappings:  # OPTIONAL, used for mentions; key: GitLab ID, val: Google Chat ID\n  5894317: 120984893489384029908  # Gandalf\n  4985120: 109238409842809234892  # Chuck Norris\n```\n\n### Config Object\n\nAlternatively, `create_app()` also accepts optional argument `config` of type\n`ggci.Config`.\n\n```python\nfrom ggci import Config, create_app\n\nconfig = Config(\n    ggci_secret='xxxxxxx',\n    user_mappings={\n        5894317: 120984893489384029908,  # Gandalf\n        4985120: 109238409842809234892,  # Chuck Norris\n    },\n)\n\napp = create_app(config=config)\n```\n\n## Features\n\n### Merge Request Events Notifications\n\nNotifications for merge requests actions.\nAll notifications for one MR are posted to the same thread (identified\nby merge request ID).\nSupported actions:\n\n- *open*: includes link with title, event author, mentions of assignees\n  and description\n- *approved*: includes link and event author\n- *unapproved*: includes link and action author\n- *update of assigness*: includes link and mentions of current assignees\n- *merged*: includes link and action author\n- *closed*: includes link and action author\n- *reopened*: includes link and action author\n",
    'author': 'Jan Lukány',
    'author_email': 'lukany.jan@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/jan.lukany/ggci',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
